@@ -21,7 +21,7 @@
 /* Even 16 color modes work (ugly colors). */
 #define VGAMODE G800x600x256
 
-#define USE_PAGEFLIPPING
+//#define USE_PAGEFLIPPING
 
 /* #define USE_SMALLOC */
 
@@ -474,7 +474,7 @@ int main(void)
 
 #ifdef USE_PAGEFLIPPING
     /* Try to enable page flipping. */
-    gl_enablepageflipping(physicalscreen);
+printf("pf=%i\n",   gl_enablepageflipping(physicalscreen));
 #endif
     setcustompalette();
     /* initfont() here caused trouble with planar 256 color modes. */
