@@ -264,12 +264,15 @@ void speed(void)
     int start_clock;
     int finish_clock;
     int diff_clock;
-    unsigned char *vgabase = vga_getgraphmem();
+    unsigned char *vgabase = vga_getgraphmem()+128*1024;
+    char *base;
 
 #ifndef LINEAR_ADDRESSING
     if (VGAMODE >= G640x480x256)
 	vga_setpage(0);
 #endif
+    
+    base=malloc(655360);
 
     start_clock = clock();
 
@@ -282,6 +285,37 @@ void speed(void)
     diff_clock = (finish_clock - start_clock)*10/CLOCKS_PER_SEC;
     printf("Video memory timing: %3d.%1ds, %dK/s\n", diff_clock / 10,
 	   (diff_clock % 10), 64*5000*10 / diff_clock);
+
+    start_clock = clock();
+
+    for (i = 0; i < 500; i++) {
+        memcpy(vgabase, base, 655360);
+    }
+
+    finish_clock = clock();
+
+    diff_clock = (finish_clock - start_clock)*10/CLOCKS_PER_SEC;
+    printf("To Video memcpy timing: %3d.%1ds, %dK/s\n", diff_clock / 10,
+	   (diff_clock % 10), 64*5000*10 / diff_clock);
+
+    start_clock = clock();
+
+    for (i = 0; i < 500; i++) {
+        unsigned char *j;
+        unsigned char *k;
+        unsigned char *l;
+        l=vgabase;
+        k=base;
+        j=base;
+        j+=655360;
+        while(k<j)*(l++)=*(k++);
+    }
+
+    finish_clock = clock();
+
+    diff_clock = (finish_clock - start_clock)*10/CLOCKS_PER_SEC;
+    printf("To Video mem copy timing: %3d.%1ds, %dK/s\n", diff_clock / 10,
+	   (diff_clock % 10), 64*5000*10 / diff_clock);
 }
 
 void sysmem_speed(void)
@@ -290,9 +324,10 @@ void sysmem_speed(void)
     int start_clock;
     int finish_clock;
     int diff_clock;
-    char *base;
+    char *base, *base2;
     
     base=malloc(655360);
+    base2=malloc(655360);
 
     start_clock = clock();
 
@@ -305,6 +340,37 @@ void sysmem_speed(void)
     diff_clock = (finish_clock - start_clock)*10/CLOCKS_PER_SEC;
     printf("System memory timing: %3d.%1ds, %dK/s\n", diff_clock / 10,
 	   (diff_clock % 10), 64*5000*10 / diff_clock);
+
+    start_clock = clock();
+
+    for (i = 0; i < 500; i++) {
+	memcpy(base, base2, 655360);
+    }
+
+    finish_clock = clock();
+
+    diff_clock = (finish_clock - start_clock)*10/CLOCKS_PER_SEC;
+    printf("To System memcpy timing: %3d.%1ds, %dK/s\n", diff_clock / 10,
+	   (diff_clock % 10), 64*5000*10 / diff_clock);
+
+    start_clock = clock();
+
+    for (i = 0; i < 500; i++) {
+        unsigned char *j;
+        unsigned char *k;
+        unsigned char *l;
+        l=base2;
+        k=base;
+        j=base;
+        j+=655360;
+        while(k<j)*(l++)=*(k++);
+    }
+
+    finish_clock = clock();
+
+    diff_clock = (finish_clock - start_clock)*10/CLOCKS_PER_SEC;
+    printf("To System mem copy timing: %3d.%1ds, %dK/s\n", diff_clock / 10,
+	   (diff_clock % 10), 64*5000*10 / diff_clock);
 }
 
 
