@@ -33,7 +33,7 @@ static inline int RGB2BGR(int c)
 /* However bswap is not supported by 386 */
 
     if (MODEFLAGS & MODEFLAG_24BPP_REVERSED)
-#ifdef __alpha__
+#ifdef NO_ASSEMBLY
 	c = ((c >> 0) & 0xff) << 16 |
 	    ((c >> 8) & 0xff) << 8 |
 	    ((c >> 16) & 0xff) << 0;
@@ -80,14 +80,9 @@ static void notimplemented(char *s)
 
 void __svgalib_driver8_setpixel(int x, int y, int c)
 {
-#ifdef __alpha__
-    vga_setcolor(c);
-    vga_drawpixel(x, y);
-#else
     char *vp;
     ASSIGNVP8(x, y, vp);
     *vp = c;
-#endif
 }
 
 void __svgalib_driver8p_setpixel(int x, int y, int c)
