@@ -1,7 +1,7 @@
 #include <linux/module.h>
 
 #include <linux/kernel.h> /* printk() */
-#include <linux/malloc.h> /* kmalloc() */
+#include <linux/slab.h> /* kmalloc() */
 #include <linux/fs.h>     /* everything... */
 #include <linux/errno.h>  /* error codes */
 #include <linux/types.h>  /* size_t */
@@ -32,10 +32,11 @@ static inline int devfs_unregister_chrdev (unsigned int major,const char *name)
 
 #include "svgalib_helper.h"
 
+#include "i810.c"
+
 int num_devices=0;
 
-static struct sh_pci_device 
-       *sh_pci_devs[SVGALIB_HELPER_MAX_NR_DEVICES];
+static struct sh_pci_device *sh_pci_devs[SVGALIB_HELPER_MAX_NR_DEVICES];
 
 #ifdef CONFIG_DEVFS_FS
 static devfs_handle_t devfs_handle;
@@ -183,6 +184,16 @@ static int svgalib_helper_ioctl( struct inode *inode, struct file *filp,
             pci_write_config_dword(pdev, pciv.address, pl);
             break;
 
+        case _IOC_NR(SVGALIB_HELPER_IOCGI810GTT):
+            i=i810_make_gtt();
+            copy_to_user((char *)arg, &i, sizeof(unsigned int));
+            break;
+        case _IOC_NR(SVGALIB_HELPER_IOCGI810GTTE):
+            copy_from_user(&i, (char *)arg, sizeof(unsigned int));
+            if(i<I810_SIZE)
+                copy_to_user((char *)arg, &i810_gttes[i], sizeof(unsigned long));
+                else return -EINVAL;
+            break;
         default: 
             return -EINVAL;
     }
@@ -292,7 +303,7 @@ int init_module(void)
      * Register your major, and accept a dynamic number
      */
      
-    printk(KERN_INFO "svgalib_helper: Initializing, version 1.9.14\n");
+    printk(KERN_INFO "svgalib_helper: Initializing, version 1.9.15\n");
     result = devfs_register_chrdev(SVGALIB_HELPER_MAJOR, "svgalib_helper", &svgalib_helper_fops);
     if (result < 0) {
         printk(KERN_WARNING "svgalib_helper: can't get major %d\n",SVGALIB_HELPER_MAJOR);
