@@ -11,6 +11,7 @@ struct sh_pci_device {
    struct pci_dev *dev;
    address_t mem[6];
    address_t len[6];
+   address_t mask[6];
    int flags[6];
 };
 
@@ -27,6 +28,12 @@ typedef struct {
    unsigned int val;
 } io_t;
 
+typedef struct {
+   int pcipos;
+   int address;
+   unsigned int val;
+} pcic_t;
+
 typedef struct {
    void *win;
    void *lfb;
@@ -41,10 +48,19 @@ typedef struct {
 #define SVGALIB_HELPER_IOCGINW		_IOW(SVGALIB_HELPER_IOC_MAGIC,5,io_t)
 #define SVGALIB_HELPER_IOCGINL		_IOW(SVGALIB_HELPER_IOC_MAGIC,6,io_t)
 
+#define SVGALIB_HELPER_IOCSPCIOUTB	_IOR(SVGALIB_HELPER_IOC_MAGIC,11,pcic_t)
+#define SVGALIB_HELPER_IOCSPCIOUTW	_IOR(SVGALIB_HELPER_IOC_MAGIC,12,pcic_t)
+#define SVGALIB_HELPER_IOCSPCIOUTL	_IOR(SVGALIB_HELPER_IOC_MAGIC,13,pcic_t)
+#define SVGALIB_HELPER_IOCGPCIINB	_IOW(SVGALIB_HELPER_IOC_MAGIC,14,pcic_t)
+#define SVGALIB_HELPER_IOCGPCIINW	_IOW(SVGALIB_HELPER_IOC_MAGIC,15,pcic_t)
+#define SVGALIB_HELPER_IOCGPCIINL	_IOW(SVGALIB_HELPER_IOC_MAGIC,16,pcic_t)
+
 #define SVGALIB_HELPER_IOCDVMA		_IO(SVGALIB_HELPER_IOC_MAGIC,7)
 #define SVGALIB_HELPER_IOCSWIND		_IOR(SVGALIB_HELPER_IOC_MAGIC,8,windowing_t)
 
 #define SVGALIB_HELPER_IOCIOPERM	_IO(SVGALIB_HELPER_IOC_MAGIC,9)
 #define SVGALIB_HELPER_IOCSREPOUTB	_IOR(SVGALIB_HELPER_IOC_MAGIC,10,io_t)
 
+#define SVGALIB_HELPER_IOCGI810GTT	_IOW(SVGALIB_HELPER_IOC_MAGIC,128,unsigned int *)
+#define SVGALIB_HELPER_IOCGI810GTTE	_IOW(SVGALIB_HELPER_IOC_MAGIC,129,unsigned int *)
 
