@@ -37,7 +37,7 @@ int main(int argc, char *argv[]){
 	if (!LRMI_init())
 		return 1;
 	ioperm(0, 0x400, 1);
-	iopl(3);
+//	iopl(3);
         i=3;
         if(argc>1){
            	sscanf(argv[1],"%i",&i);
