@@ -35,7 +35,6 @@ there are still the following problems:
 #include "vgaregs.h"
 #include "vgammvgaio.h"
 #include "interface.h"
-#include "accel.h"
 
 #define APMREG_SAVE(i) (VGA_TOTAL_REGS+i)
 #define APM_TOTAL_REGS (VGA_TOTAL_REGS + 38)
@@ -43,8 +42,6 @@ there are still the following problems:
 static int apm_init(int, int, int);
 static void apm_unlock(void);
 
-void __svgalib_apmaccel_init(AccelSpecs * accelspecs, int bpp, int width_in_pixels);
-
 static int apm_memory,apm_chiptype;
 static int apm_is_linear,apm_xbase;
 
@@ -372,8 +369,6 @@ static int apm_setmode(int mode, int prv_mode)
 
     if (IS_IN_STANDARD_VGA_DRIVER(mode)) {
 
-/*	__svgalib_clear_accelspecs(__svgalib_driverspecs->accelspecs);
-*/
 	/* Let the standard VGA driver set standard VGA modes */
 	/* But first reset an Cirrus extended register that */
 	/* an old XFree86 Trident probe corrupts. */
@@ -400,8 +395,6 @@ static int apm_setmode(int mode, int prv_mode)
     apm_setregs(moderegs, mode);	/* Set extended regs. */
     free(moderegs);
 
-    __svgalib_InitializeAcceleratorInterface(modeinfo);
-
     free(modeinfo);
     return 0;
 }
