@@ -15,7 +15,6 @@
 #include "timing.h"
 #include "vgaregs.h"
 #include "interface.h"
-#include "accel.h"
 #include "vgapci.h"
 
 #define BANSHEEREG_SAVE(i) (VGA_TOTAL_REGS+i)
@@ -25,8 +24,6 @@ static int banshee_init(int, int, int);
 static void banshee_unlock(void);
 static void banshee_lock(void);
 
-void __svgalib_bansheeaccel_init(AccelSpecs * accelspecs, int bpp, int width_in_pixels);
-
 static int banshee_memory, chiptype;
 static int banshee_is_linear, banshee_linear_base, banshee_io_base;
 
@@ -316,11 +313,8 @@ static int banshee_setmode(int mode, int prv_mode)
     banshee_setregs(moderegs, mode);		/* Set extended regs. */
     free(moderegs);
 
-    __svgalib_InitializeAcceleratorInterface(modeinfo);
-
     free(modeinfo);
 
-
     return 0;
 }
 
