@@ -20,7 +20,6 @@ TODO: SDRAM, reference frequency checking.
 #include "timing.h"
 #include "vgaregs.h"
 #include "interface.h"
-#include "accel.h"
 #include "vgapci.h"
 #include "mga.h"
 #define SECONDCRTC 0
@@ -36,8 +35,6 @@ static int g400_init(int, int, int);
 static void g400_unlock(void);
 static void g400_lock(void);
 
-void __svgalib_g400accel_init(AccelSpecs * accelspecs, int bpp, int width_in_pixels);
-
 static int g400_memory, id, g400_pciposition;
 static int g400_is_linear, g400_linear_base, g400_mmio_base;
 
