@@ -16,7 +16,6 @@
 #include "timing.h"
 #include "vgaregs.h"
 #include "interface.h"
-#include "accel.h"
 #include "vgapci.h"
 #include "i740_reg.h"
 #include "vgammvgaio.h"
@@ -61,8 +60,6 @@ static int i740_init(int, int, int);
 static void i740_unlock(void);
 static void i740_lock(void);
 
-void __svgalib_i740accel_init(AccelSpecs * accelspecs, int bpp, int width_in_pixels);
-
 static int i740_memory, I740HasSGRAM;
 static int i740_is_linear, i740_linear_base, i740_mmio_base;
 
@@ -588,8 +585,6 @@ static int i740_setmode(int mode, int prv_mode)
     i740_setregs(moderegs, mode);		/* Set extended regs. */
     free(moderegs);
 
-    __svgalib_InitializeAcceleratorInterface(modeinfo);
-
     free(modeinfo);
     return 0;
 }
