@@ -1,9 +1,10 @@
 /*
  i810 chipset driver
   written by Matan Ziv-Av.
-  Only tested on a Hercules Terminator 2x/I
-  Interlaced modes don't work yet.
 */
+
+#define USE_GTT 1
+
 #include <stdlib.h>
 #include <stdio.h>		
 #include <string.h>
@@ -16,11 +17,12 @@
 #include "timing.h"
 #include "vgaregs.h"
 #include "interface.h"
-#include "accel.h"
 #include "vgapci.h"
 #include "i810_reg.h"
 #include "i810_wmark.c"
 #include "vgammvgaio.h"
+#include <sys/ioctl.h>
+#include "svgalib_helper.h"
 
 typedef struct {
 /* 00 */
@@ -69,13 +71,39 @@ static void lock(void);
 
 static int memory, lmfreq;
 static int is_linear, linear_base, mmio_base;
-
+static unsigned int cur_base;
 static CardSpecs *cardspecs;
 
+#ifdef USE_GTT
+static void setpage(int page)
+{
+	static int oldpage = 0;
+
+	if (page != oldpage)
+	{
+		if (page)
+			mmap(BANKED_POINTER,
+			     __svgalib_banked_mem_size,
+			     PROT_READ | PROT_WRITE,
+			     MAP_SHARED | MAP_FIXED,
+			     __svgalib_mem_fd, 
+                             __svgalib_banked_mem_base +  (page << 16));
+		else
+			mmap(BANKED_POINTER,
+			     __svgalib_banked_mem_size,
+			     PROT_READ | PROT_WRITE,
+			     MAP_SHARED | MAP_FIXED,
+			     __svgalib_mem_fd,
+			     __svgalib_banked_mem_base);
+		oldpage = page;
+	}
+}
+#else /* USE_GTT */
 static void setpage(int page)
 {
    __svgalib_outgra(0x11,page);
 }
+#endif
 
 static int __svgalib_inlinearmode(void)
 {
@@ -96,9 +124,11 @@ static void getmodeinfo(int mode, vga_modeinfo *modeinfo)
     modeinfo->flags &= ~HAVE_RWPAGE;
 
     if (modeinfo->bytesperpixel >= 1) {
+#ifdef USE_GTT
 	if(linear_base)modeinfo->flags |= CAPABLE_LINEAR;
         if (__svgalib_inlinearmode())
 	    modeinfo->flags |= IS_LINEAR;
+#endif
     }
 }
 
@@ -156,7 +186,11 @@ static void setregs(const unsigned char regs[], int mode)
     unlock();		
     restore = (vgaI810Ptr)(regs+VGA_TOTAL_REGS);
 
-#if 1
+    temp=__svgalib_inseq(0x01);
+    __svgalib_outseq(0, 1);
+    __svgalib_outseq(1, temp | 0x20);
+
+    usleep(50000);
 
     temp=INREG8(DRAM_ROW_CNTL_HI);
     temp &= ~0x18;
@@ -164,12 +198,10 @@ static void setregs(const unsigned char regs[], int mode)
 
     usleep(1000); /* Wait 1 ms */
 
-#endif
-
     /* Write the M, N and P values */
     OUTREG16(VCLK2_VCO_M, restore->VideoClk2_M);
     OUTREG16(VCLK2_VCO_N, restore->VideoClk2_N);
-    OUTREG(VCLK2_VCO_DIV_SEL, restore->VideoClk2_DivisorSel);
+    OUTREG8(VCLK2_VCO_DIV_SEL, restore->VideoClk2_DivisorSel);
 
 
     temp = INREG(PIXPIPE_CONFIG_0);
@@ -177,21 +209,18 @@ static void setregs(const unsigned char regs[], int mode)
     temp |= (restore->PixelPipeCfg0 & (DAC_8_BIT | HW_CURSOR_ENABLE));
     OUTREG(PIXPIPE_CONFIG_0, temp);
 
+    temp = __svgalib_incrtc(IO_CTNL);
+    temp &= ~(EXTENDED_ATTR_CNTL|EXTENDED_CRTC_CNTL); 
+    temp |= EXTENDED_CRTC_CNTL;
+    __svgalib_outcrtc(IO_CTNL, temp);
 
     __svgalib_outcrtc(EXT_VERT_TOTAL, restore->ExtVertTotal);
-
     __svgalib_outcrtc(EXT_VERT_DISPLAY, restore->ExtVertDispEnd);
-
     __svgalib_outcrtc(EXT_VERT_SYNC_START, restore->ExtVertSyncStart);
-
     __svgalib_outcrtc(EXT_VERT_BLANK_START, restore->ExtVertBlankStart);
-
     __svgalib_outcrtc(EXT_HORIZ_TOTAL, restore->ExtHorizTotal);
-
     __svgalib_outcrtc(EXT_HORIZ_BLANK, restore->ExtHorizBlank);
-
     __svgalib_outcrtc(EXT_OFFSET, restore->ExtOffset);
-
     __svgalib_outcrtc(EXT_START_ADDR, restore->ExtStartAddr);
     __svgalib_outcrtc(EXT_START_ADDR_HI, restore->ExtStartAddrHi);
 
@@ -217,13 +246,12 @@ static void setregs(const unsigned char regs[], int mode)
     temp &= ~0x18;
     temp |= 0x08;
     OUTREG8(DRAM_ROW_CNTL_HI, temp);
-#if 0
 
+#if 0
     temp = INREG8(BITBLT_CNTL);
     temp &= ~COLEXP_MODE;
     temp |= restore->BitBLTControl;
     OUTREG8(BITBLT_CNTL, temp);
-
 #endif
 
     temp = INREG8(DISPLAY_CNTL);
@@ -246,22 +274,22 @@ static void setregs(const unsigned char regs[], int mode)
     temp |= restore->PixelPipeCfg1;
     OUTREG8(PIXPIPE_CONFIG_1, temp);
 
-
 #if 0
-
     temp = INREG(FWATER_BLC);
     temp &= ~(LM_BURST_LENGTH | LM_FIFO_WATERMARK | MM_BURST_LENGTH | MM_FIFO_WATERMARK);
     temp |= restore->LMI_FIFO_Watermark;
     OUTREG(FWATER_BLC, temp);
 #endif
 
+    temp=__svgalib_inseq(0x01);
+    __svgalib_outseq(1, temp & ~0x20);
+    __svgalib_outseq(0, 3);
+
     temp = __svgalib_incrtc(IO_CTNL);
     temp &= ~(EXTENDED_ATTR_CNTL|EXTENDED_CRTC_CNTL); 
     temp |= restore->IOControl;
     __svgalib_outcrtc(IO_CTNL, temp);
 
-    usleep(50000);
-
 }
 
 /* Return nonzero if mode is available */
@@ -293,65 +321,57 @@ static int modeavailable(int mode)
     return SVGADRV;
 }
 
-#define MAX_VCO_FREQ 600.0
+#define MAX_VCO_FREQ 600000
 #define TARGET_MAX_N 30
-#define REF_FREQ 24.0
+#define REF_FREQ 24000
 
 #define CALC_VCLK(m,n,p) \
-    (double)m / ((double)n * (1 << p)) * 4 * REF_FREQ
+    ((m * 4 * REF_FREQ / n ) >> p) 
 
 static void
-I810CalcVCLK(double freq ,int *M, int *N, int *DIVSEL )
+I810CalcVCLK(int freq, int *M, int *N, int *DIVSEL )
 {
     int m, n, p;
-    double f_out;
-    double f_err, f_best;
-    double f_vco;
+    int f_out;
+    int f_err, f_best;
+    int f_vco;
     int m_best = 0, n_best = 0, p_best = 0;
-    double f_target = freq;
-    double err_max = 0.005;
-    double err_target = 0.001;
-    double err_best = 999999.0;
+    int f_target = freq;
+    int err_max = 5000;
+    int err_target = 100;
+    int err_best = 999999;
 
-    p_best = p = log(MAX_VCO_FREQ/f_target)/log((double)2);
+    p=1;
+    while((1<<p)*freq*2 < MAX_VCO_FREQ) p++;
+    if(p>5)p=5;
+    p_best=p;
 
     f_vco = f_target * (1 << p);
 
     n = 2;
     do {
-	n++;
-	m = f_vco / (REF_FREQ / (double)n) / (double)4.0 + 0.5;
-	if (m < 3) m = 3;
-	f_out = CALC_VCLK(m,n,p);
-	f_err = 1.0 - (f_target/f_out);
-	if (fabs(f_err) < err_max) {
-	    m_best = m;
-	    n_best = n;
+        n++;
+        m = f_vco * n / REF_FREQ / 4;
+        if (m < 3) m = 3;
+        f_out = CALC_VCLK(m,n,p);
+        f_err = f_target-f_out;
+        if (f_err < err_max) {
+            m_best = m;
+            n_best = n;
             f_best = f_out;
-	    err_best = f_err;
-	}
-    } while ((fabs(f_err) >= err_target) &&
-	     ((n <= TARGET_MAX_N) || (fabs(err_best) > err_max)));
+            err_best = f_err;
+        }
+    } while ((f_err >= err_target) &&
+             ((n <= TARGET_MAX_N) || (err_best > err_max)));
 
-    if (fabs(f_err) < err_target) {
-	m_best = m;
+    if (f_err < err_target) {
+        m_best = m;
         n_best = n;
     }
 
     *M     = (m_best-2) & 0x3FF;
     *N     = (n_best-2) & 0x3FF;
     *DIVSEL= p_best << 4;
-
-#if 0
-    fprintf(stderr,"Setting dot clock to %.6lf MHz "
-	   "[ %02X %02X %02X ] "
-	   "[ %d %d %d %d ]\n",
-	   CALC_VCLK(m_best,n_best,p_best,d_best),
-	   new->VideoClk2_M,
-	   new->VideoClk2_N,
-	   new->VideoClk2_DivisorSel,
-	   m_best, n_best, p_best, d_best);
-#endif
 }
 
 /* Local, called by setmode(). */
@@ -402,11 +422,14 @@ static void initializemode(unsigned char *moderegs,
     new->IOControl = EXTENDED_CRTC_CNTL;
 
     /* Turn on linear and page mapping */
-    new->AddressMapping = PACKED_MODE_ENABLE |/* LINEAR_MODE_ENABLE | */ PAGE_MAPPING_ENABLE;
-
-    /* Turn on GUI mode */
-    new->DisplayControl = /* HIRES_MODE */ 2;
-
+#ifdef USE_GTT
+    new->AddressMapping =  LINEAR_MODE_ENABLE | 8 ;
+    new->DisplayControl =  1;
+#else
+    new->AddressMapping = PACKED_MODE_ENABLE | PAGE_MAPPING_ENABLE ;
+    new->DisplayControl =  2;
+#endif
+    
     /* Calculate the extended CRTC regs */
     new->ExtVertTotal = (modetiming->CrtcVTotal - 2) >> 8;
     new->ExtVertDispEnd = (modetiming->CrtcVDisplay - 1) >> 8;
@@ -423,7 +446,7 @@ static void initializemode(unsigned char *moderegs,
 
     moderegs[VGA_AR11] = 0;
 
-    I810CalcVCLK(modetiming->pixelClock/1000, &m, &n, &divsel);
+    I810CalcVCLK(modetiming->pixelClock, &m, &n, &divsel);
     
     new->VideoClk2_M = m;
     new->VideoClk2_N = n; 
@@ -443,6 +466,7 @@ static void initializemode(unsigned char *moderegs,
     return ;
 }
 
+static void setdisplaystart(int address);
 
 static int setmode(int mode, int prv_mode)
 {
@@ -474,7 +498,7 @@ static int setmode(int mode, int prv_mode)
     setregs(moderegs, mode);		/* Set extended regs. */
     free(moderegs);
 
-    __svgalib_InitializeAcceleratorInterface(modeinfo);
+    setdisplaystart(0);
 
     free(modeinfo);
     return 0;
@@ -485,8 +509,7 @@ static int setmode(int mode, int prv_mode)
 
 static void unlock(void)
 {
-    __svgalib_outcrtc(0x11,__svgalib_incrtc(0x11)&0x7f);
-    
+    __svgalib_outcrtc(0x11,__svgalib_incrtc(0x11)&0x7f);    
 }
 
 static void lock(void)
@@ -495,7 +518,10 @@ static void lock(void)
 
 
 #define VENDOR_ID 0x8086
-#define ID 0x1132
+#define ID10	0x7121
+#define ID10DC	0x7123
+#define ID10E	0x7125
+#define ID15	0x1132
 
 /* Indentify chipset, initialize and return non-zero if detected */
 
@@ -503,10 +529,12 @@ static int test(void)
 {
     int found;
     long buf[64];
+    int id;
     
     found=__svgalib_pci_find_vendor_vga(VENDOR_ID,buf,0);
+    id=buf[0]>>16;
     
-    if(!found&&(((buf[0]>>16)&0xffff)==ID)){
+    if( !found && ((id==ID10) || (id==ID10DC) || (id==ID10E) || (id==ID15)) ){
           init(0,0,0);
           return 1;
     };
@@ -526,16 +554,19 @@ static void setdisplaystart(int address)
   __svgalib_outcrtc(EXT_START_ADDR,((address&0x3f0000)>>16)|EXT_START_ADDR_ENABLE);
 }
 
-
 /* Set logical scanline length (usually multiple of 8) */
-/* Cirrus supports multiples of 8, up to 4088 */
 
 static void setlogicalwidth(int width)
 {   
+#ifdef USE_GTT
     int offset = width >> 3;
+#else
+    int offset = width >> 4;
+#endif
  
     __svgalib_outcrtc(0x13,offset&0xff);
     __svgalib_outcrtc(EXT_OFFSET, offset>>8);
+
 }
 
 static int linear(int op, int param)
@@ -553,7 +584,7 @@ static int cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
     int i, j, tmp;
     unsigned long *b3;
     unsigned long l1, l2, l3, l4;
-    
+
     switch(cmd){
         case CURSOR_INIT:
             return 1;
@@ -570,18 +601,17 @@ static int cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
             OUTREG8(CURSOR_Y_HI,(p2&0x700)>>8);
             break;
         case CURSOR_SELECT:
-            i=memory*1024-(p1+1)*4096;
-            if(i>4096*1024)i-=4096*1024;
             tmp=INREG8(PIXPIPE_CONFIG_0);
             OUTREG8(PIXPIPE_CONFIG_0,tmp| EXTENDED_PALETTE);
-            __svgalib_outpal(4,cur_colors[p1*6],cur_colors[p1*6+1],cur_colors[p1*6+2]);
-            __svgalib_outpal(5,cur_colors[p1*6+3],cur_colors[p1*6+4],cur_colors[p1*6+5]);
+            OUTREG8(0x3c6, 0xff);
+            OUTREG8(0x3c8, 0x04);
+            for(i=0;i<6;i++) 
+                OUTREG8(0x3c9, cur_colors[p1*6+i]);
             OUTREG8(PIXPIPE_CONFIG_0,tmp);
-            OUTREG(CURSOR_BASEADDR,i); 
+            OUTREG(CURSOR_BASEADDR,cur_base+(15-p1)*256); 
             break;
         case CURSOR_IMAGE:
-            i=memory*1024-(p1+1)*4096;
-            if(i>4096*1024)i-=4096*1024;
+            i=memory*1024-(p1+1)*256;
             b3=(unsigned long *)p5;
             switch(p2) {
                 case 0:
@@ -663,16 +693,20 @@ DriverSpecs __svgalib_i810_driverspecs =
 static int init(int force, int par1, int par2)
 {
     unsigned long buf[64];
+    unsigned int t, i, id;
     int found=0;
     
     found=__svgalib_pci_find_vendor_vga(VENDOR_ID,buf,0);
     linear_base=0;
-    if(!found&&(((buf[0]>>16)&0xffff)==ID)){
+    id=buf[0]>>16;
+    
+    if( !found && ((id==ID10) || (id==ID10DC) || (id==ID10E) || (id==ID15)) ){
        linear_base=buf[4]&0xffffff00;
        mmio_base=buf[5]&0xffffff00;
-    };
+    } else return -1; 
+    ;
 
-    __svgalib_vgammbase=mmap(0,0x1000,PROT_READ|PROT_WRITE,MAP_SHARED,__svgalib_mem_fd,mmio_base);
+    __svgalib_vgammbase=mmap(0,0x20000,PROT_READ|PROT_WRITE,MAP_SHARED,__svgalib_mem_fd,mmio_base);
     __svgalib_mm_io_mapio();
     
     if (force) {
@@ -681,6 +715,29 @@ static int init(int force, int par1, int par2)
         memory = 512;
     };
 
+#ifdef USE_GTT
+    if(*(unsigned int *)(__svgalib_vgammbase + 0x2020) & 1) {
+       /* GTT already set */
+       memory = 4096;
+    } else {
+        memory = 4096;
+        ioctl(__svgalib_mem_fd, SVGALIB_HELPER_IOCGI810GTT, &t);
+        *(unsigned int *)(__svgalib_vgammbase + 0x2020) = t | 1;
+        for(i=0;i<1024;i++) {
+            t=i;
+            ioctl(__svgalib_mem_fd, SVGALIB_HELPER_IOCGI810GTTE, &t);
+            *(unsigned int *)(__svgalib_vgammbase + 0x10000 + 4*i) = t;
+        }
+    }
+    cur_base = (memory>>2)-1;
+    ioctl(__svgalib_mem_fd, SVGALIB_HELPER_IOCGI810GTTE, &cur_base);
+    cur_base &= ~1 ;
+    __svgalib_banked_mem_base=linear_base;
+#else
+    linear_base=0;
+    __svgalib_banked_mem_base=0xa0000;
+#endif
+
     if(buf[20]&0x10) {
         lmfreq=133;
     } else {
@@ -698,15 +755,14 @@ static int init(int force, int par1, int par2)
     cardspecs->maxPixelClock16bpp = 163000;	
     cardspecs->maxPixelClock24bpp = 128000;
     cardspecs->maxPixelClock32bpp = 86000;
-    cardspecs->flags = CLOCK_PROGRAMMABLE;
+    cardspecs->flags = CLOCK_PROGRAMMABLE | INTERLACE_DIVIDE_VERT;
     cardspecs->maxHorizontalCrtc = 4088;
     cardspecs->maxPixelClock4bpp = 0;
-    cardspecs->nClocks =0;
+    cardspecs->nClocks = 0;
     cardspecs->mapClock = map_clock;
     cardspecs->mapHorizontalCrtc = map_horizontal_crtc;
     cardspecs->matchProgrammableClock=match_programmable_clock;
     __svgalib_driverspecs = &__svgalib_i810_driverspecs;
-    __svgalib_banked_mem_base=0xa0000;
     __svgalib_banked_mem_size=0x10000;
     __svgalib_linear_mem_base=linear_base;
     __svgalib_linear_mem_size=memory*0x400;
