@@ -20,7 +20,6 @@ Problems:
 #include "timing.h"
 #include "vgaregs.h"
 #include "interface.h"
-#include "accel.h"
 #include "vgapci.h"
 
 #include "lagunaio.h"
@@ -57,8 +56,6 @@ static int laguna_init(int, int, int);
 static void laguna_unlock(void);
 static void laguna_lock(void);
 
-void __svgalib_lagunaaccel_init(AccelSpecs * accelspecs, int bpp, int width_in_pixels);
-
 static int laguna_memory, laguna_chiptype;
 static int laguna_is_linear, laguna_linear_base, laguna_mmio_base;
 
@@ -411,8 +408,6 @@ static int laguna_setmode(int mode, int prv_mode)
     laguna_setregs(moderegs, mode);		/* Set extended regs. */
     free(moderegs);
 
-    __svgalib_InitializeAcceleratorInterface(modeinfo);
-
     free(modeinfo);
     return 0;
 }
@@ -452,8 +447,6 @@ static int laguna_test(void)
 
 
 /* Set display start address (not for 16 color modes) */
-/* Cirrus supports any address in video memory (up to 2Mb) */
-
 static void laguna_setdisplaystart(int address)
 { 
     address=address >> 2;
@@ -466,8 +459,6 @@ static void laguna_setdisplaystart(int address)
 
 
 /* Set logical scanline length (usually multiple of 8) */
-/* Cirrus supports multiples of 8, up to 4088 */
-
 static void laguna_setlogicalwidth(int width)
 {   
     int offset = width >> 3;
