@@ -12,7 +12,6 @@ Matrox Millennium / Millenium II driver
 #include "timing.h"
 #include "vgaregs.h"
 #include "interface.h"
-#include "accel.h"
 #include "vgapci.h"
 #include "vgammvgaio.h"
 #include "mga.h"
@@ -21,8 +20,6 @@ static int mil_init(int, int, int);
 static void mil_unlock(void);
 static void mil_lock(void);
 
-void __svgalib_skaccel_init(AccelSpecs * accelspecs, int bpp, int width_in_pixels);
-
 static int mil_memory=0, mil_pciposition;
 static int mil_is_linear, mil_linear_base, mil_mmio_base, mil_chiptype;
 static int interleave, rounding;
@@ -708,8 +705,6 @@ static int mil_setmode(int mode, int prv_mode)
     mil_setregs(moderegs, mode);		/* Set extended regs. */
     free(moderegs);
 
-    __svgalib_InitializeAcceleratorInterface(modeinfo);
-
     if(mode>=G640x480x256)
         for(i=0;i<256;i++) __svgalib_outpal(i,i,i,i);
 
