@@ -25,7 +25,6 @@ when entering text mode) or to the et4000 autodetection.
 #include "timing.h"
 #include "vgaregs.h"
 #include "interface.h"
-#include "accel.h"
 #include "vgapci.h"
 
 #define MXREG_SAVE(i) (VGA_TOTAL_REGS+i)
@@ -43,8 +42,6 @@ static int mx_init(int, int, int);
 static void mx_unlock(void);
 static void mx_lock(void);
 
-void __svgalib_mxaccel_init(AccelSpecs * accelspecs, int bpp, int width_in_pixels);
-
 static int mx_memory,mx_chiptype;
 static int mx_is_linear, mx_linear_base;
 static int SysControl, BankIdx;
@@ -382,9 +379,6 @@ static int mx_setmode(int mode, int prv_mode)
     mx_setregs(moderegs, mode);		/* Set extended regs. */
     free(moderegs);
 
-    __svgalib_InitializeAcceleratorInterface(modeinfo);
-
-
     free(modeinfo);
     return 0;
 }
