@@ -189,7 +189,7 @@ static int nv3_modeavailable(int mode)
     return SVGADRV;
 }
 
-static int NV3ClockSelect(float clockIn,float *clockOut,int *mOut,
+static int CalcVClock(int clockIn,int *clockOut,int *mOut,
                                         int *nOut,int *pOut);
 
 static int CalculateCRTC(ModeTiming *mode, ModeInfo *modeinfo, unsigned char *moderegs)
@@ -288,15 +288,15 @@ static void nv3_initializemode(unsigned char *moderegs,
     unsigned int config0=0;
     
     int m,n,p;
-    float clockIn=(float)modetiming->pixelClock;
-    float clockOut;
+    int clockIn=modetiming->pixelClock;
+    int clockOut;
     int pixelDepth;
   
     nv3_saveregs(moderegs);
 
     __svgalib_setup_VGA_registers(moderegs, modetiming, modeinfo);
 
-    if(!NV3ClockSelect(clockIn,&clockOut,&m,&n,&p)) {return ;}
+    CalcVClock(clockIn,&clockOut,&m,&n,&p);
 
     *(unsigned int *)(moderegs+REG(12)) = PRAMDAC_Val(VPLL_COEFF_NDIV,n) | 
         				  PRAMDAC_Val(VPLL_COEFF_MDIV,m) | 
@@ -330,7 +330,9 @@ static void nv3_initializemode(unsigned char *moderegs,
         PRAMDAC_Val(GENERAL_CONTROL_565_MODE,modeinfo->greenWeight==6)|
         PRAMDAC_Def(GENERAL_CONTROL_TERMINATION,37OHM)|
         ((modeinfo->bitsPerPixel>8) ? 
-            PRAMDAC_Def(GENERAL_CONTROL_BPC,8BITS) : PRAMDAC_Def(GENERAL_CONTROL_BPC,6BITS)) |          PRAMDAC_Def(GENERAL_CONTROL_VGA_STATE,SEL); /* Not sure about this */
+            PRAMDAC_Def(GENERAL_CONTROL_BPC,8BITS) : 
+            PRAMDAC_Def(GENERAL_CONTROL_BPC,6BITS)) | PRAMDAC_Def(GENERAL_CONTROL_VGA_STATE,SEL); 
+            /* Not sure about this */
 
     switch(chip){
       case Riva128:
@@ -730,11 +732,15 @@ static int nv3_init(int force, int par1, int par2)
              PLL_INPUT_FREQ= (temp&0x20) ? 14318 : 13500;
              MAXVCLOCK=256000;
              P_MAX=4; /* XFree say 3, but 4 works on my Riva128 */
+             if(PLL_INPUT_FREQ==13500)M_MAX=12; else M_MAX=13;
+             M_MIN=M_MAX-5;
              break;
           case RivaTNT:
              PLL_INPUT_FREQ= (temp&0x40) ? 14318 : 13500;
              MAXVCLOCK=350000;
              P_MAX=4;
+             if(PLL_INPUT_FREQ==13500)M_MAX=13; else M_MAX=14;
+             M_MIN=M_MAX-6;
              break;
           case GEFORCE:
           default:
@@ -742,12 +748,10 @@ static int nv3_init(int force, int par1, int par2)
                  	     (temp&0x40  ) ? 14318 : 13500;
              MAXVCLOCK=350000;
              P_MAX=4;
+             if(PLL_INPUT_FREQ==13500)M_MAX=13; else M_MAX=14;
+             M_MIN=M_MAX-6;
              break;
        };
-
-       if(PLL_INPUT_FREQ==14318)M_MIN=7; else M_MIN=8;
-       M_MAX=M_MIN+6;
-       if(chip==Riva128)M_MAX--;
     };
 
     if (__svgalib_driver_report) {
@@ -787,37 +791,50 @@ static int nv3_init(int force, int par1, int par2)
     return 0;
 }
 
-static int NV3ClockSelect(float clockIn,float *clockOut,int *mOut,
-                                        int *nOut,int *pOut)
-{ 
-  int m,n,p;
-  float bestDiff=1e10;
-  float target=0.0;
-  float best=0.0;
-  float diff;
-  int nMax,nMin;
+/*
+ * Calculate the Video Clock parameters for the PLL.
+ */
+static int CalcVClock
+(
+    int           clockIn,
+    int          *clockOut,
+    int          *mOut,
+    int          *nOut,
+    int          *pOut
+)
+{
+    unsigned DeltaNew, DeltaOld;
+    unsigned VClk, Freq;
+    unsigned M, N, P;
+    
+    DeltaOld = 0xFFFFFFFF;
 
-  *clockOut=0.0;
-  for(p=P_MIN;p<=P_MAX;p++) {
-    for(m=M_MIN;m<=M_MAX;m++) {
-      float fm=(float)m;
-      /* Now calculate maximum and minimum values for n */
-      nMax=(int) (((MAXVCLOCK/PLL_INPUT_FREQ)*fm)-0.5);
-      nMin=(int) (((128000/PLL_INPUT_FREQ)*fm)+0.5);
-      n=(int)(((clockIn*((float)(1<<p)))/PLL_INPUT_FREQ)*fm);
-      if(n>=nMin && n<=nMax) {  
-        float fn=(float)n;
-        target=(PLL_INPUT_FREQ*(fn/fm))/((float)(1<<p));
-        diff=fabs(target-clockIn);
-        if(diff<bestDiff) {
-          bestDiff=diff;
-          best=target;
-          *mOut=m;*nOut=n;*pOut=p;
-          *clockOut=best;
-	}
-      }
+    VClk     = (unsigned)clockIn;
+    
+    for (P = 0; P <= P_MAX; P ++)
+    {
+        Freq = VClk << P;
+        if ((Freq >= 128000) && (Freq <= MAXVCLOCK))
+        {
+            for (M = M_MIN; M <= M_MAX; M++)
+            {
+                N    = ((VClk<<P) * M + PLL_INPUT_FREQ/2) / PLL_INPUT_FREQ;
+                Freq = (PLL_INPUT_FREQ * N / M) >> P;
+                if (Freq > VClk)
+                    DeltaNew = Freq - VClk;
+                else
+                    DeltaNew = VClk - Freq;
+                if ((DeltaNew < DeltaOld) && (N<256))
+                {
+                    *mOut     = M;
+                    *nOut     = N;
+                    *pOut     = P;
+                    *clockOut = Freq;
+                    DeltaOld  = DeltaNew;
+                }
+            }
+        }
     }
-  }
-  return (best!=0.0);    
+    return (DeltaOld != 0xFFFFFFFF);
 }
 
