@@ -14,11 +14,12 @@ Rage 128 chipset driver
 #include "timing.h"
 #include "vgaregs.h"
 #include "interface.h"
-#include "accel.h"
 #include "vgapci.h"
 #include "endianess.h"
 #include "r128_reg.h"
 
+static enum { Rage128=0, Radeon } chiptype; /* r128io needs to know */
+static int dac6bits;
 #include "r128io.h"
 
 #ifdef __PPC
@@ -58,6 +59,7 @@ typedef struct {
     CARD32     config_cntl;
     CARD32     mem_vga_wp_sel;
     CARD32     mem_vga_rp_sel;
+    CARD32     surface_cntl;
 				/* Other registers to save for VT switches */
     CARD32     dp_datatype;
     CARD32     gen_reset_cntl;
@@ -191,6 +193,7 @@ static void R128RestoreCommonRegisters(R128SavePtr restore)
     OUTREG(R128_CONFIG_CNTL,          restore->config_cntl);
     OUTREG(R128_MEM_VGA_WP_SEL,	      restore->mem_vga_wp_sel);
     OUTREG(R128_MEM_VGA_RP_SEL,	      restore->mem_vga_rp_sel);
+    OUTREG(RADEON_SURFACE_CNTL,	      restore->surface_cntl);
 
 }
 
@@ -259,6 +262,55 @@ static void R128PLLWriteUpdate(void)
 }
 
 /* Write PLL registers. */
+static void RADEONRestorePLLRegisters(R128SavePtr restore)
+{
+    OUTPLLP(R128_VCLK_ECP_CNTL,
+	    RADEON_VCLK_SRC_SEL_CPUCLK,
+	    ~(RADEON_VCLK_SRC_SEL_MASK));
+
+    OUTPLLP(R128_PPLL_CNTL,
+	    R128_PPLL_RESET
+	    | R128_PPLL_ATOMIC_UPDATE_EN
+	    | R128_PPLL_VGA_ATOMIC_UPDATE_EN,
+	    ~(R128_PPLL_RESET
+	      | R128_PPLL_ATOMIC_UPDATE_EN
+	      | R128_PPLL_VGA_ATOMIC_UPDATE_EN));
+
+    OUTREGP(R128_CLOCK_CNTL_INDEX,
+	    R128_PLL_DIV_SEL,
+	    ~(R128_PLL_DIV_SEL));
+
+    OUTPLLP(R128_PPLL_REF_DIV,
+	    restore->ppll_ref_div,
+	    ~R128_PPLL_REF_DIV_MASK);
+
+    OUTPLLP(R128_PPLL_DIV_3,
+	    restore->ppll_div_3,
+	    ~R128_PPLL_FB3_DIV_MASK);
+
+    OUTPLLP(R128_PPLL_DIV_3,
+	    restore->ppll_div_3,
+	    ~R128_PPLL_POST3_DIV_MASK);
+
+    R128PLLWriteUpdate();
+    R128PLLWaitForReadUpdateComplete();
+
+    OUTPLL(R128_HTOTAL_CNTL, restore->htotal_cntl);
+
+    OUTPLLP(R128_PPLL_CNTL,
+	    0,
+	    ~(R128_PPLL_RESET
+	      | R128_PPLL_SLEEP
+	      | R128_PPLL_ATOMIC_UPDATE_EN
+	      | R128_PPLL_VGA_ATOMIC_UPDATE_EN));
+
+    usleep(5000); /* Let the clock to lock */
+
+    OUTPLLP(R128_VCLK_ECP_CNTL,
+	    RADEON_VCLK_SRC_SEL_PPLLCLK,
+	    ~(RADEON_VCLK_SRC_SEL_MASK));
+}
+
 static void R128RestorePLLRegisters(R128SavePtr restore)
 {
     OUTREGP(R128_CLOCK_CNTL_INDEX, R128_PLL_DIV_SEL, 0xffff);
@@ -304,7 +356,7 @@ static void R128RestorePalette( R128SavePtr restore)
 {
     int           i;
 
-    if (!restore->palette_valid) return;
+if (!restore->palette_valid) return;
 
     /* Select palette 0 (main CRTC) if using FP-enabled chip */
 //    if (info->HasPanelRegs) PAL_SELECT(0);
@@ -331,9 +383,19 @@ static void R128RestoreMode(R128SavePtr restore)
 //    if (info->HasPanelRegs)
 //	R128RestoreFPRegisters(restore);
 //    if (!info->HasPanelRegs || info->CRTOnly)
-    R128RestorePLLRegisters(restore);
-    R128RestoreDDARegisters(restore);
-    R128RestorePalette(restore);
+    switch(chiptype) {
+        case Rage128:
+            R128RestorePLLRegisters(restore);
+            break;
+        case Radeon:
+            RADEONRestorePLLRegisters(restore);
+            break;
+    }
+            
+    if(chiptype == Rage128) {
+	R128RestoreDDARegisters(restore);
+    }
+    	R128RestorePalette(restore);
 }
 
 /* Read common registers. */
@@ -355,6 +417,8 @@ static void R128SaveCommonRegisters(R128SavePtr save)
     save->config_cntl        = INREG(R128_CONFIG_CNTL);
     save->mem_vga_wp_sel     = INREG(R128_MEM_VGA_WP_SEL);
     save->mem_vga_rp_sel     = INREG(R128_MEM_VGA_RP_SEL);
+	if(chiptype==Radeon)
+    	save->surface_cntl   = INREG(RADEON_SURFACE_CNTL);
 }
 
 /* Read CRTC registers. */
@@ -428,7 +492,8 @@ static void R128SaveMode(R128SavePtr save)
 //    if (R128PTR(pScrn)->HasPanelRegs)
 //	R128SaveFPRegisters(save);
     R128SavePLLRegisters(save);
-    R128SaveDDARegisters(save);
+	if(chiptype == Rage128)
+    	R128SaveDDARegisters(save);
     R128SavePalette(save);
 
     save->dp_datatype      = INREG(R128_DP_DATATYPE);
@@ -456,6 +521,10 @@ static void R128InitCommonRegisters(R128SavePtr save)
     save->mem_vga_rp_sel     = 0;
     save->config_cntl        = INREG(R128_CONFIG_CNTL);
     save->bus_cntl           = BusCntl;
+    if(chiptype == Radeon) {
+        if(save->bus_cntl & RADEON_BUS_READ_BURST) 
+            save->bus_cntl |=RADEON_BUS_RD_DISCARD_EN;
+    }
 
     save->amcgpio_en_reg   = INREG(R128_AMCGPIO_EN_REG);
     save->amcgpio_mask     = INREG(R128_AMCGPIO_MASK);
@@ -480,7 +549,6 @@ static Bool R128InitCrtcRegisters(R128SavePtr save,
     int    hsync_fudge_default[] = { 0x00, 0x12, 0x09, 0x09, 0x06, 0x05 };
     int    hsync_fudge_fp[]      = { 0x12, 0x11, 0x09, 0x09, 0x05, 0x05 };
     int    hsync_fudge_fp_crt[]  = { 0x12, 0x10, 0x08, 0x08, 0x04, 0x04 };
-    int    dac6bits;
 
     dac6bits=0;
 
@@ -515,6 +583,7 @@ static Bool R128InitCrtcRegisters(R128SavePtr save,
 			     : 0));
 
     save->crtc_ext_cntl = R128_VGA_ATI_LINEAR | R128_XCRT_CNT_EN | R128_VGA_MEM_PS_EN;
+    if(chiptype == Radeon) save->crtc_ext_cntl |= R128_CRTC_CRT_ON;
     save->dac_cntl      = (R128_DAC_MASK_ALL
 			   | R128_DAC_VGA_ADR_EN
 			   | (dac6bits ? 0 : R128_DAC_8BIT_EN));
@@ -561,6 +630,11 @@ static Bool R128InitCrtcRegisters(R128SavePtr save,
     save->crtc_pitch       = info->width / 8;
 
     save->config_cntl |= R128_CFG_VGA_RAM_EN;
+    
+    if(chiptype == Radeon) {
+        save->crtc_pitch |= save->crtc_pitch<<16;
+        save->surface_cntl = RADEON_SURF_TRANSLATION_DIS;        
+    }
 
 #ifdef __PPC
     /* Change the endianness of the aperture */
@@ -673,6 +747,50 @@ static void R128InitFPRegisters(R128SavePtr orig, R128SavePtr save,
 #endif
 
 /* Define PLL registers for requested video mode. */
+static void RADEONInitPLLRegisters(R128SavePtr save, R128PLLPtr pll,
+				 double dot_clock)
+{
+    unsigned long freq = dot_clock * 100;
+    struct {
+	int divider;
+	int bitvalue;
+    } *post_div,
+      post_divs[]   = {
+				/* From RAGE 128 VR/RAGE 128 GL Register
+				   Reference Manual (Technical Reference
+				   Manual P/N RRG-G04100-C Rev. 0.04), page
+				   3-17 (PLL_DIV_[3:0]).  */
+	{  1, 0 },              /* VCLK_SRC                 */
+	{  2, 1 },              /* VCLK_SRC/2               */
+	{  4, 2 },              /* VCLK_SRC/4               */
+	{  8, 3 },              /* VCLK_SRC/8               */
+
+	{  3, 4 },              /* VCLK_SRC/3               */
+	{ 16, 5 },
+        {  6, 6 },              /* VCLK_SRC/6               */
+	{ 12, 7 },              /* VCLK_SRC/12              */
+	{  0, 0 }
+    };
+
+    if (freq > pll->max_pll_freq)      freq = pll->max_pll_freq;
+    if (freq * 12 < pll->min_pll_freq) freq = pll->min_pll_freq / 12;
+
+    for (post_div = &post_divs[0]; post_div->divider; ++post_div) {
+	save->pll_output_freq = post_div->divider * freq;
+	if (save->pll_output_freq >= pll->min_pll_freq
+	    && save->pll_output_freq <= pll->max_pll_freq) break;
+    }
+
+    save->dot_clock_freq = freq;
+    save->feedback_div   = R128Div(pll->reference_div * save->pll_output_freq,
+				   pll->reference_freq);
+    save->post_div       = post_div->divider;
+
+    save->ppll_ref_div   = pll->reference_div;
+    save->ppll_div_3     = (save->feedback_div | (post_div->bitvalue << 16));
+    save->htotal_cntl    = 0;
+}
+
 static void R128InitPLLRegisters(R128SavePtr save, R128PLLPtr pll,
 				 double dot_clock)
 {
@@ -789,11 +907,19 @@ static Bool R128Init(ModeTiming *mode, ModeInfo *info, R128SavePtr save)
     if (HasPanelRegs)
 	R128InitFPRegisters(&info->SavedReg, save, mode, info);
 #endif
-    R128InitPLLRegisters(save, &pll, dot_clock);
+    switch(chiptype) {
+        case Rage128:
+            R128InitPLLRegisters(save, &pll, dot_clock);
+            break;
+        case Radeon:
+            RADEONInitPLLRegisters(save, &pll, dot_clock);
+            break;
+    }
+    
     if (!R128InitDDARegisters(save, &pll, info))
 	return 0;
 //    if (!info->PaletteSavedOnVT) 
-   		R128InitPalette(save);
+    R128InitPalette(save);
 
     return 1;
 }
@@ -802,13 +928,35 @@ static int r128_init(int, int, int);
 static void r128_unlock(void);
 static void r128_lock(void);
 
-void __svgalib_skaccel_init(AccelSpecs * accelspecs, int bpp, int width_in_pixels);
-
-static int r128_memory, r128_chiptype;
+static int r128_memory;
 static int r128_is_linear, r128_linear_base, r128_mmio_base;
 
 static CardSpecs *cardspecs;
 
+static void radeon_setpage(int page)
+{
+	static int oldpage = 0;
+
+	if (page != oldpage)
+	{
+		if (page)
+			mmap(BANKED_POINTER,
+			     __svgalib_banked_mem_size,
+			     PROT_READ | PROT_WRITE,
+			     MAP_SHARED | MAP_FIXED,
+			     __svgalib_mem_fd, 
+                             __svgalib_banked_mem_base +  (page << 16));
+		else
+			mmap(BANKED_POINTER,
+			     __svgalib_banked_mem_size,
+			     PROT_READ | PROT_WRITE,
+			     MAP_SHARED | MAP_FIXED,
+			     __svgalib_mem_fd,
+			     __svgalib_banked_mem_base);
+		oldpage = page;
+	}
+}
+
 static void r128_setpage(int page)
 {
     page*=2;
@@ -936,16 +1084,12 @@ static int r128_setmode(int mode, int prv_mode)
     r128_setregs(moderegs, mode);		/* Set extended regs. */
     free(moderegs);
 
-    __svgalib_InitializeAcceleratorInterface(modeinfo);
-
-
     free(modeinfo);
     return 0;
 }
 
 
 /* Unlock chipset-specific registers */
-
 static void r128_unlock(void)
 {
     __svgalib_outcrtc(0x11, __svgalib_incrtc(0x11) & 0x7f);    
@@ -955,23 +1099,10 @@ static void r128_lock(void)
 {
 }
 
-
-#define VENDOR_ID 0x1002
-
 /* Indentify chipset, initialize and return non-zero if detected */
-
 static int r128_test(void)
 {
-    int found, id;
-    unsigned long buf[64];
-    
-    found=__svgalib_pci_find_vendor_vga(VENDOR_ID,buf,0);
-
-    if(found) return 0;
-    id=(buf[0]>>16)&0xffff;
-    if((id>=0x5350) || (id<=0x4fff)) return 0;
-    
-    r128_init(0,0,0);
+    return r128_init(0,0,0);
     return 1;
 }
 
@@ -1148,19 +1279,19 @@ DriverSpecs __svgalib_r128_driverspecs =
     r128_cursor,
 };
 
-/* Initialize chipset (called after detection) */
+#define VENDOR_ID 0x1002
 
+/* Initialize chipset (called after detection) */
 static int r128_init(int force, int par1, int par2)
 {
     unsigned long buf[64];
-    int found=0;
-#ifndef NO_BIOS
+    int found=0, id, NOBIOS=0;
     unsigned char *BIOS_POINTER;
-#endif
+    char chipnames[2][9]={"Rage 128", "Radeon"};
 
     if (force) {
 	r128_memory = par1;
-        r128_chiptype = par2;
+        chiptype = par2;
     } else {
 
     };
@@ -1168,68 +1299,100 @@ static int r128_init(int force, int par1, int par2)
     found=__svgalib_pci_find_vendor_vga(VENDOR_ID,buf,0);
 
     if(found) return 0;
-    r128_chiptype=(buf[0]>>16)&0xffff;
-    if((r128_chiptype>=0x5350) || (r128_chiptype<=0x4fff)) return 0;
-
+    
+    chiptype=-1;
+    id=(buf[0]>>16)&0xffff;
+    
+    if( (id==0x4c45) ||
+        (id==0x4c56) ||
+        (id==0x4d46) ||
+        (id==0x4d4c) ||
+        ((id>>8)==0x50) ||
+        ((id>>8)==0x52) ||
+        ((id>>8)==0x53) ||
+        ((id>>8)==0x54))
+        chiptype=Rage128;
+        
+    if( (id==0x4242) ||
+        (id==0x4c57) ||
+        (id==0x4c59) ||
+        (id==0x4c5a) ||
+        ((id>>8)==0x51))
+        chiptype = Radeon;
+	
+    if(chiptype==-1) return 0;
+    
     r128_linear_base=buf[4]&0xffffff00;
     r128_mmio_base=buf[6]&0xffffff00;
 
     MMIO_POINTER = mmap(0, 16*1024, PROT_READ | PROT_WRITE, MAP_SHARED, __svgalib_mem_fd,
        			r128_mmio_base);
-#ifndef NO_BIOS
-    BIOS_POINTER = mmap(0, 64*1024, PROT_READ | PROT_WRITE, MAP_SHARED, __svgalib_mem_fd,
-       			0xc0000);
-#endif
 
     r128_memory      = INREG(R128_CONFIG_MEMSIZE) / 1024;
     BusCntl            = INREG(R128_BUS_CNTL);
     HasPanelRegs	= 0;
     CRTOnly		= 1;
     r128_ramtype	= 1;
-#ifdef NO_BIOS
-    {
-    int x_mpll_ref_fb_div, xclk_cntl, Nx, M;
-    int PostDivSet[] = {0, 1, 2, 4, 8, 3, 6, 12};
-    
-    pll.reference_freq = 2950;
-    pll.min_pll_freq   = 12500;
-    pll.max_pll_freq   = 25000;
-    pll.reference_div =	INPLL(R128_PPLL_REF_DIV) & R128_PPLL_REF_DIV_MASK;
 
-    x_mpll_ref_fb_div = INPLL(R128_X_MPLL_REF_FB_DIV);
-    xclk_cntl = INPLL(R128_XCLK_CNTL) & 0x7;
-    Nx = (x_mpll_ref_fb_div & 0x00FF00) >> 8;
-    M =  (x_mpll_ref_fb_div & 0x0000FF);
-    pll.xclk =  R128Div((2 * Nx * pll.reference_freq),
-                          (M * PostDivSet[xclk_cntl]));
-    }
-#else
+#ifndef __PPC    
+    if(__svgalib_secondary) 
+#endif
+        NOBIOS=1;
+
+    if(NOBIOS) {
+        int x_mpll_ref_fb_div, xclk_cntl, Nx, M;
+        int PostDivSet[] = {0, 1, 2, 4, 8, 3, 6, 12};
+    
+        switch(chiptype) {
+            case Rage128:
+                pll.reference_freq = 2950;
+                pll.min_pll_freq   = 12500;
+                pll.max_pll_freq   = 25000;
+                pll.reference_div =	INPLL(R128_PPLL_REF_DIV) & R128_PPLL_REF_DIV_MASK;
+    
+                x_mpll_ref_fb_div = INPLL(R128_X_MPLL_REF_FB_DIV);
+                xclk_cntl = INPLL(R128_XCLK_CNTL) & 0x7;
+                Nx = (x_mpll_ref_fb_div & 0x00FF00) >> 8;
+                M =  (x_mpll_ref_fb_div & 0x0000FF);
+                pll.xclk =  R128Div((2 * Nx * pll.reference_freq),
+                                      (M * PostDivSet[xclk_cntl]));
+                break;
+            case Radeon:
+                pll.reference_freq = 2700;
+                pll.min_pll_freq   = 12500;
+                pll.max_pll_freq   = 35000;
+                pll.reference_div  = 67;
+                pll.xclk 	   = 16667;
+                break;
+        }
+    } else 
 #define R128_BIOS16(x) (*(unsigned short *)(BIOS_POINTER + x))
 #define R128_BIOS32(x) (*(unsigned int *)(BIOS_POINTER + x))
     {
-    CARD16 bios_header    = R128_BIOS16(0x48);
-    CARD16 pll_info_block = R128_BIOS16(bios_header + 0x30);
-    pll.reference_freq = R128_BIOS16(pll_info_block + 0x0e);
-    pll.reference_div  = R128_BIOS16(pll_info_block + 0x10);
-    pll.min_pll_freq   = R128_BIOS32(pll_info_block + 0x12);
-    pll.max_pll_freq   = R128_BIOS32(pll_info_block + 0x16);
-    pll.xclk           = R128_BIOS16(pll_info_block + 0x08);
+        CARD16 bios_header;
+        CARD16 pll_info_block;
+        BIOS_POINTER = mmap(0, 64*1024, PROT_READ | PROT_WRITE, MAP_SHARED, __svgalib_mem_fd,
+       			    0xc0000);
+        bios_header    = R128_BIOS16(0x48);
+        pll_info_block = R128_BIOS16(bios_header + 0x30);
+        pll.reference_freq = R128_BIOS16(pll_info_block + 0x0e);
+        pll.reference_div  = R128_BIOS16(pll_info_block + 0x10);
+        pll.min_pll_freq   = R128_BIOS32(pll_info_block + 0x12);
+        pll.max_pll_freq   = R128_BIOS32(pll_info_block + 0x16);
+        pll.xclk           = R128_BIOS16(pll_info_block + 0x08);
+        munmap(BIOS_POINTER, 64*1024);
     }
-#endif
-#if 0
+#if 1
 fprintf(stderr,"pll: %i %i %i %i %i\n",pll.reference_freq,pll.reference_div,
     pll.min_pll_freq,    pll.max_pll_freq, pll.xclk);
 #endif
 
     munmap(MMIO_POINTER, 16*1024);
-#ifndef NO_BIOS
-    munmap(BIOS_POINTER, 64*1024);
-#endif
 
     r128_mapio();
-    
+	
     if (__svgalib_driver_report) {
-	fprintf(stderr,"Using Rage 128 driver, %iKB.\n",r128_memory);
+	fprintf(stderr,"Using ATI %s driver, %iKB.\n",chipnames[chiptype],r128_memory);
     };
 
     cardspecs = malloc(sizeof(CardSpecs));
@@ -1253,5 +1416,10 @@ fprintf(stderr,"pll: %i %i %i %i %i\n",pll.reference_freq,pll.reference_div,
     __svgalib_linear_mem_size=r128_memory*0x400;
     __svgalib_mmio_base=r128_mmio_base;
     __svgalib_mmio_size=16*1024;
-    return 0;
+    if(chiptype==Radeon) {
+        __svgalib_banked_mem_base=r128_linear_base;
+        __svgalib_r128_driverspecs.__svgalib_setpage = radeon_setpage;
+    }
+
+    return 1;
 }
