@@ -1487,4 +1487,14 @@
 #define R128_CCE_VC_CNTL_PRIM_WALK_RING          0x00000030
 #define R128_CCE_VC_CNTL_NUM_SHIFT               16
 
+#define RADEON_SURFACE_CNTL			 0x0b00
+
+#define RADEON_VCLK_SRC_SEL_MASK		 0x0003
+#define RADEON_VCLK_SRC_SEL_CPUCLK		 0x0000
+#define RADEON_VCLK_SRC_SEL_PPLLCLK		 0x0003
+
+#define RADEON_BUS_READ_BURST			 (1<<30)
+#define RADEON_BUS_RD_DISCARD_EN		 (1<<24)
+
+#define RADEON_SURF_TRANSLATION_DIS		 (1<<8)
 #endif
