@@ -180,7 +180,7 @@ static char* mouse_process_option(int option, int mode, char **nptr) {
  int newmtype;
 
 #ifdef DEBUG_ACCEL
-  printf("Processing option %ld (%s)\n", (long)option, mouse_config_options[option]);
+  fprintf(stderr,"Processing option %ld (%s)\n", (long)option, mouse_config_options[option]);
 #endif
 
    switch (option) {
@@ -265,7 +265,7 @@ static char* mouse_process_option(int option, int mode, char **nptr) {
 #ifdef ALLOW_MOUSE_OVERRIDE
 	newmtype = vga_getmousetype();
 	if (m_type != newmtype) {
-	    printf("svgalib: mouse-init: mouse type override %d to %d\n",
+	    fprintf(stderr,"svgalib: mouse-init: mouse type override %d to %d\n",
 		   m_type, newmtype);
 	    m_type = newmtype;
 	}
@@ -397,14 +397,14 @@ static int ms_init(void)
         } else {
          int i;
 #ifdef DEBUG_ACCEL
-	  printf("m_accel_powertable:\n");
+	  fprintf(stderr,"m_accel_powertable:\n");
 #endif
           for (i = 1; i < m_accel_thresh; i++) {
             m_accel_powertable[i] = pow((double)(i-1)/(m_accel_thresh-1), m_accel_power)
               * (m_accel_mult - m_accel_offset) + m_accel_offset;
 
 #ifdef DEBUG_ACCEL
-	    printf("  %ld => %f (%f)\n", (long)i, (double)m_accel_powertable[i] * i,
+	    fprintf(stderr,"  %ld => %f (%f)\n", (long)i, (double)m_accel_powertable[i] * i,
 	  			     (double)m_accel_powertable[i]);
 #endif
           }
@@ -413,14 +413,14 @@ static int ms_init(void)
     }
 
 #ifdef DEBUG_ACCEL
-    printf("m_accel_type: %ld\n",	(long)m_accel_type);
-    printf("m_force: %ld\n",		(long)m_force);
-    printf("m_accel_thresh: %ld\n",	(long)m_accel_thresh);
-    printf("m_accel_offset: %ld\n",	(long)m_accel_offset);
-    printf("m_accel_mult: %f\n",	(double)m_accel_mult);
-    printf("m_accel_power: %f\n",	(double)m_accel_power);
-    printf("m_maxdelta: %ld\n",		(long)m_maxdelta);
-    printf("m_accel_maxdelta: %ld\n",	(long)m_accel_maxdelta);
+    fprintf(stderr,"m_accel_type: %ld\n",	(long)m_accel_type);
+    fprintf(stderr,"m_force: %ld\n",		(long)m_force);
+    fprintf(stderr,"m_accel_thresh: %ld\n",	(long)m_accel_thresh);
+    fprintf(stderr,"m_accel_offset: %ld\n",	(long)m_accel_offset);
+    fprintf(stderr,"m_accel_mult: %f\n",	(double)m_accel_mult);
+    fprintf(stderr,"m_accel_power: %f\n",	(double)m_accel_power);
+    fprintf(stderr,"m_maxdelta: %ld\n",		(long)m_maxdelta);
+    fprintf(stderr,"m_accel_maxdelta: %ld\n",	(long)m_accel_maxdelta);
 #endif
 
     /*  Ensure that the open will get a file descriptor greater
@@ -627,6 +627,7 @@ static inline void ms_close(void)
 	ioctl(__svgalib_mouse_fd, TIOCMSET, &m_old_modem_info);
 
     close(__svgalib_mouse_fd);
+    mouse_open=0;
     __svgalib_mouse_fd = -1;
 }
 
@@ -676,7 +677,7 @@ static int get_ms_event(int wait) {
 	nu_bytes += bytesread;
 
 #ifdef DEBUG
-    printf("#bytes in buffer: %d\n", nu_bytes);
+    fprintf(stderr,"#bytes in buffer: %d\n", nu_bytes);
 #endif
 
   handle_packets:
@@ -684,10 +685,10 @@ static int get_ms_event(int wait) {
     /* Handle packets in buffer. */
 
 #ifdef DEBUG
-    printf("Bytes left in buffer: %d at %d, packet is %d bytes\n",
+    fprintf(stderr,"Bytes left in buffer: %d at %d, packet is %d bytes\n",
 	   nu_bytes - i, i, proto[m_type][4]);
     if (nu_bytes - i > 0 )
-      printf("Header byte: %c %d\n", (buf[i] & 0177), buf[i]);
+      fprintf(stderr,"Header byte: %c %d\n", (buf[i] & 0177), buf[i]);
 
 #endif
 
@@ -747,7 +748,7 @@ static int get_ms_event(int wait) {
     if ((buf[i] & proto[m_type][0]) != proto[m_type][1]) {
 	/* Not a header byte. */
 #ifdef DEBUG
-    printf("Bad header byte: %c %d\n", (buf[i] & 0177), buf[i]);
+    fprintf(stderr,"Bad header byte: %c %d\n", (buf[i] & 0177), buf[i]);
 #endif
 	i++;
 	goto handle_packets;
@@ -1103,7 +1104,7 @@ static int get_ms_event(int wait) {
 	    /*    if (fabs(dx) < sorb_trans_thresh[1]) dx = 0; */
             i+=13;
 #ifdef DEBUG
-    printf("Got D packet! but=%d, x=%d y=%d z=%d rx=%d ry=%d rz=%d\n",
+    fprintf(stderr,"Got D packet! but=%d, x=%d y=%d z=%d rx=%d ry=%d rz=%d\n",
             but,dx,dy,dz,drx,dry,drz);
 #endif
             break;
@@ -1113,23 +1114,23 @@ static int get_ms_event(int wait) {
 	    if (but==MOUSE_RESETBUTTON)
 	      mouse_orientation=1-mouse_orientation;
 #ifdef DEBUG
-    printf("Got K packet! but=%d, x=%d y=%d z=%d rx=%d ry=%d rz=%d\n",
+    fprintf(stderr,"Got K packet! but=%d, x=%d y=%d z=%d rx=%d ry=%d rz=%d\n",
             but,dx,dy,dz,drx,dry,drz);
 #endif
 	    i+=6;
 	    break;
 	  case 'R':
 #ifdef DEBUG
-    printf("Got init string!\n");
+    fprintf(stderr,"Got init string!\n");
 #endif
 	    for (j=i;((buf[j] !=13)&&(j<nu_bytes));j++)
-              printf("%c",(buf[j] & 0177));
-            printf("\n\n");
+              fprintf(stderr,"%c",(buf[j] & 0177));
+            fprintf(stderr,"\n\n");
             i=j+1;
             break;
 	  default:
 #ifdef DEBUG
-    printf("Got unknown packet!\n");
+    fprintf(stderr,"Got unknown packet!\n");
 #endif
             i++;
             break;
@@ -1171,20 +1172,20 @@ static int get_ms_event(int wait) {
 
       case MOUSE_ACCEL_TYPE_NORMAL:
 #ifdef DEBUG_ACCEL 
-  printf("%ld\t", (long)dx);
+  fprintf(stderr,"%ld\t", (long)dx);
 #endif
 	if (abs(dx) > m_accel_thresh) dx = (int) ((float)dx * m_accel_mult);
 	if (abs(dy) > m_accel_thresh) dy = (int) ((float)dy * m_accel_mult);
 	if (abs(dz) > m_accel_thresh) dz = (int) ((float)dz * m_accel_mult);
 #ifdef DEBUG_ACCEL
-  printf("%ld\n", (long)dx);
+  fprintf(stderr,"%ld\n", (long)dx);
 #endif
         break;
 
 
       case MOUSE_ACCEL_TYPE_POWER:
 #ifdef DEBUG_ACCEL 
-  printf("%ld\t", (long)dx);
+  fprintf(stderr,"%ld\t", (long)dx);
 #endif
 	delta = abs(dx);
 	dx = (delta >= m_accel_thresh)  ?  (float)dx * m_accel_mult :
@@ -1199,7 +1200,7 @@ static int get_ms_event(int wait) {
 	     (float)dz * m_accel_powertable[delta];
 
 #ifdef DEBUG_ACCEL
-  printf("%ld\n", (long)dx);
+  fprintf(stderr,"%ld\n", (long)dx);
 #endif
         break;
     }
