@@ -113,6 +113,9 @@ extern DacMethods __svgalib_IBMRGB52x_methods;
 #ifdef INCLUDE_SC1148X_DAC
 extern DacMethods __svgalib_SC1148X_methods;
 #endif
+#ifdef INCLUDE_ICS_GENDAC_DAC
+extern DacMethods __svgalib_ICS_GENDAC_methods;
+#endif
 
 extern DacMethods *__svgalib_all_dacs[];	/* List of all defined DACs. */
 
@@ -134,9 +137,6 @@ int __svgalib_setDacSpeed(int dacspeed, int defspeed);
 void __svgalib_Sierra_32K_savestate(unsigned char *regs);
 void __svgalib_Sierra_32K_restorestate(const unsigned char *regs);
 
-int __svgalib_att20c490_map_clock(int bpp, int pixelclock);
-int __svgalib_att20c490_map_horizontal_crtc(int bpp, int pixelclock, int htiming);
-
 #ifdef __OPTIMIZE__
 static __inline__ void _ramdac_dactocomm(void)
 {
