@@ -502,7 +502,7 @@ void __svgalib_readmodes(FILE * inp, ModeTable ** mt, int *dac, unsigned *clocks
     p = modes;
     while (p != NULL) {
 #ifdef DEBUG
-	printf("Found mode %2d: %s\n", p->mnum, mode2name(p->x, p->y, p->c));
+	fprintf(stderr,"Found mode %2d: %s\n", p->mnum, mode2name(p->x, p->y, p->c));
 #endif
 	(*mt)[i].mode_number = p->mnum;
 	(*mt)[i].regs = p->regs;
