@@ -1,4 +1,3 @@
-#undef GTF
 /* 
  * Generic mode timing module.
  */
@@ -111,11 +110,9 @@ MonitorModeTiming __svgalib_standard_timings[] =
 
 static MonitorModeTiming *user_timings = NULL;
 static MonitorModeTiming *current_timing, *force_timing = NULL, new_timing;
-#ifdef GTF /* Ineed to get rid of the libm.so requirement */
 static void GTF_calcTimings(double hPixels,double vLines,double freq,
         int type,int wantMargins,int wantInterlace, int wantDblscan, 
         MonitorModeTiming *mmt);
-#endif
 /*
  * SYNC_ALLOWANCE is in percent
  * 1% corresponds to a 315 Hz deviation at 31.5 kHz, 1 Hz at 100 Hz
@@ -141,7 +138,7 @@ static int timing_within_monitor_spec(MonitorModeTiming * mmtp)
     if ((mmtp->flags & DOUBLESCAN))
 	vsf /= 2.0f;
 #ifdef DEBUG
-    printf("hsf = %f (in:%d), vsf = %f (in:%d)\n",
+    fprintf(stderr,"hsf = %f (in:%d), vsf = %f (in:%d)\n",
 	   hsf / 1000, (int) INRANGE(hsf, horizsync),
 	   vsf, (int) INRANGE(vsf, vertrefresh));
 #endif
@@ -229,6 +226,7 @@ static MonitorModeTiming *search_mode(MonitorModeTiming * timings,
     for (t = timings; t; t = t->next)
 	if (t->HDisplay == modeinfo->width
 	    && t->VDisplay == modeinfo->height
+            && ( (!(t->flags&INTERLACED)) || (!(cardspecs->flags&NO_INTERLACE)) )
 	    && timing_within_monitor_spec(t)
 	    && t->pixelClock <= maxclock
 	    && t->pixelClock > bestclock
@@ -528,11 +526,11 @@ currently only works for x:y = 4:3, clue means:
 */
 
    MonitorModeTiming mmt, *bestmodetiming = NULL ;
-   int bestx,besty /*, bestclock */ ;
+   int bestx, besty, flag, mx, my /*, bestclock */ ;
 
    int aspect_ratio=1000*y/x;
    switch(clue) {
-      case 0:
+      case 0: /* 0,1 only 4:3 ratio, find close mode, and up/down scale timing */
       case 1:
           if((aspect_ratio>765)||(aspect_ratio<735))return 0;
           if(clue==0)find_up_timing(x,y,&bestx,&besty,&bestmodetiming);
@@ -554,10 +552,15 @@ currently only works for x:y = 4:3, clue means:
              return 1;
           };
           break;
-      case 256:
-      case 257: {
-          int mx;
+      case 2: /* Use GTF, caller provides all parameters. */
+          flag = arg>>16;
+          GTF_calcTimings(x , y, arg&0xffff, flag&3, flag&4, flag&8, flag&16, &mmt); 
+          __svgalib_addusertiming(&mmt);
+          return 1;
+      
           
+      case 256: /* 256,257: find a 4:3 mode with y close to requested, and */
+      case 257: /* up/down scale timing */
           mx=y*4/3;
           if((clue&1)==0)find_up_timing(mx,y,&bestx,&besty,&bestmodetiming);
           if((clue&1)==1)find_down_timing(mx,y,&bestx,&besty,&bestmodetiming);
@@ -577,11 +580,9 @@ currently only works for x:y = 4:3, clue means:
              __svgalib_addusertiming(&mmt);
              return 1;
           };
-      };
-      case 258:
-      case 259: {
-          int my;
-          
+          break;
+      case 258: /* 258,259: find a 4:3 mode with x close to requested, and */
+      case 259: /* up/down scale timing */
           my=(x*3)>>2;
           if((clue&1)==0)find_up_timing(x,my,&bestx,&besty,&bestmodetiming);
           if((clue&1)==1)find_down_timing(x,my,&bestx,&besty,&bestmodetiming);
@@ -601,13 +602,11 @@ currently only works for x:y = 4:3, clue means:
              __svgalib_addusertiming(&mmt);
              return 1;
           };
-      };
+          break;
    };
    return 0;
 };
 
-#ifdef GTF
-
 /* Everything from here to the end of the file is copyright by 
 scitechsoft. See their original program in utils/gtf subdirectory */
    
@@ -893,4 +892,3 @@ static void GTF_calcTimings(double hPixels,double vLines,double freq,
                             | ((wantDblscan) ? DOUBLESCAN : 0);
 }
 
-#endif
