@@ -110,6 +110,8 @@ typedef struct {
 #define GREATER_1024_DIVIDE_VERT	0x4
 /* The DAC doesn't support 64K colors (5-6-5) at 16bpp, just 5-5-5. */
 #define NO_RGB16_565			0x8
+/* Card (or driver) can't do interlaced modes */
+#define NO_INTERLACE			0x10
 
 /* Mode info. */
 typedef struct {
