@@ -657,6 +657,11 @@ DriverSpecs *__svgalib_driverspecslist[] =
 #else
     NULL,
 #endif
+#ifdef INCLUDE_PM2_DRIVER
+    &__svgalib_pm2_driverspecs,
+#else
+    NULL,
+#endif
 };
 
 static char *driver_names[] =
@@ -698,6 +703,7 @@ static char *driver_names[] =
 "TRIDENT",
 "RENDITION",
 "G450C2",
+"PM2",
  NULL};
 
 /* Chipset drivers */
@@ -1035,6 +1041,11 @@ int __svgalib_getchipset(void)
 	    CHIPSET = G400;
 	else
 #endif
+#ifdef INCLUDE_PM2_DRIVER_TEST
+	if (__svgalib_pm2_driverspecs.test())
+	    CHIPSET = PM2;
+	else
+#endif
 #ifdef INCLUDE_SAVAGE_DRIVER_TEST
 	if (__svgalib_savage_driverspecs.test())
 	    CHIPSET = SAVAGE;
