@@ -104,7 +104,7 @@ static int check_owner(int vc)
     if ((stat(fname, &sbuf) >= 0) && (getuid() == sbuf.st_uid)) {
         return 1;
     }
-    printf("You must be the owner of the current console to use svgalib.\n");
+    fprintf(stderr,"You must be the owner of the current console to use svgalib.\n");
     return 0;
 }
 
@@ -145,7 +145,7 @@ void __svgalib_open_devconsole(void)
     }
 
     if ((__svgalib_tty_fd = open("/dev/console", O_RDWR)) < 0) {
-        printf("svgalib: can't open /dev/console \n");
+        fprintf(stderr,"svgalib: can't open /dev/console \n");
         exit(1);
     }
     if (ioctl(__svgalib_tty_fd, VT_OPENQRY, &__svgalib_vc) < 0)
@@ -164,7 +164,7 @@ void __svgalib_open_devconsole(void)
             goto error;
         /* success, redirect all stdios */
         if (DREP)
-            printf("[svgalib: allocated virtual console #%d]\n", __svgalib_vc);
+            fprintf(stderr,"[svgalib: allocated virtual console #%d]\n", __svgalib_vc);
         fflush(stdin);
         fflush(stdout);
         fflush(stderr);
@@ -187,7 +187,7 @@ error:
     if (__svgalib_tty_fd > 2)
 	close(__svgalib_tty_fd);
     __svgalib_tty_fd = - 1;
-    printf("Not running in a graphics capable console,\n"
+    fprintf(stderr,"Not running in a graphics capable console,\n"
 	 "and unable to find one.\n");
     }
 }
