@@ -12,6 +12,7 @@
 #include "vga.h"
 #include "libvga.h"
 #include "vgabg.h"
+#include "ppcmemset.h"
 
 int vga_clear(void)
 {
@@ -23,6 +24,7 @@ int vga_clear(void)
     case G320x240x256:
     case G320x400x256:
     case G360x480x256:
+    case G400x300x256X:
       modeX:
         
 	/* write to all planes */
@@ -45,19 +47,10 @@ int vga_clear(void)
 	    {
 		int i;
 		int pages = (CI.ydim * CI.xbytes + 65535) >> 16;
-#if defined(CONFIG_ALPHA_JENSEN)
-		int j;
-#endif
 		for (i = 0; i < pages; ++i) {
 		    vga_setpage(i);
-
-#if defined(CONFIG_ALPHA_JENSEN)
-		    for (j = 0; j < 65536; j += 2)
-			gr_writew(0, j);
-#else
 		    /* clear video memory */
-		    memset(GM, 0, 65536);
-#endif
+                    memset(GM, 0, 65536);
 		}
 	    }
 	    break;
