@@ -72,7 +72,7 @@ int vga_setpalette(int index, int red, int green, int blue)
 
     if (__svgalib_grayscale) {
 	if ((unsigned) index >= sizeof(__svgalib_green_backup) / sizeof(__svgalib_green_backup[0])) {
-	    printf("vga_setpalette: color index %d out of range\n", index);
+	    fprintf(stderr,"vga_setpalette: color index %d out of range\n", index);
 	}
 	__svgalib_green_backup[index] = green;
 
@@ -97,7 +97,7 @@ int vga_getpalette(int index, int *red, int *green, int *blue)
     else get_lut(index, red, green, blue);
     if (__svgalib_grayscale) {
 	if ((unsigned) index >= sizeof(__svgalib_green_backup) / sizeof(__svgalib_green_backup[0])) {
-	    printf("vga_getpalette: color index %d out of range\n", index);
+	    fprintf(stderr,"vga_getpalette: color index %d out of range\n", index);
 	}
 	*green = __svgalib_green_backup[index];
     }
